(function() {
    "use strict";
    function $$morph$range$utils$$clear(parentNode, firstNode, lastNode) {
      if (!parentNode) { return; }

      var node = firstNode;
      var nextNode;
      do {
        nextNode = node.nextSibling;
        parentNode.removeChild(node);
        if (node === lastNode) {
          break;
        }
        node = nextNode;
      } while (node);
    }

    function $$morph$range$utils$$insertBefore(parentNode, firstNode, lastNode, refNode) {
      var node = firstNode;
      var nextNode;
      do {
        nextNode = node.nextSibling;
        parentNode.insertBefore(node, refNode);
        if (node === lastNode) {
          break;
        }
        node = nextNode;
      } while (node);
    }

    // constructor just initializes the fields
    // use one of the static initializers to create a valid morph.
    function $$morph$range$$Morph(domHelper, contextualElement) {
      this.domHelper = domHelper;
      // context if content if current content is detached
      this.contextualElement = contextualElement;
      // inclusive range of morph
      // these should be nodeType 1, 3, or 8
      this.firstNode = null;
      this.lastNode  = null;

      // flag to force text to setContent to be treated as html
      this.parseTextAsHTML = false;

      // morph list graph
      this.parentMorphList = null;
      this.previousMorph   = null;
      this.nextMorph       = null;
    }

    $$morph$range$$Morph.empty = function (domHelper, contextualElement) {
      var morph = new $$morph$range$$Morph(domHelper, contextualElement);
      morph.clear();
      return morph;
    };

    $$morph$range$$Morph.create = function (domHelper, contextualElement, node) {
      var morph = new $$morph$range$$Morph(domHelper, contextualElement);
      morph.setNode(node);
      return morph;
    };

    $$morph$range$$Morph.attach = function (domHelper, contextualElement, firstNode, lastNode) {
      var morph = new $$morph$range$$Morph(domHelper, contextualElement);
      morph.setRange(firstNode, lastNode);
      return morph;
    };

    $$morph$range$$Morph.prototype.setContent = function Morph$setContent(content) {
      if (content === null || content === undefined) {
        return this.clear();
      }

      var type = typeof content;
      switch (type) {
        case 'string':
          if (this.parseTextAsHTML) {
            return this.domHelper.setMorphHTML(this, content);
          }
          return this.setText(content);
        case 'object':
          if (typeof content.nodeType === 'number') {
            return this.setNode(content);
          }
          /* Handlebars.SafeString */
          if (typeof content.toHTML === 'function') {
            return this.setHTML(content.toHTML());
          }
          if (this.parseTextAsHTML) {
            return this.setHTML(content.toString());
          }
          /* falls through */
        case 'boolean':
        case 'number':
          return this.setText(content.toString());
        default:
          throw new TypeError('unsupported content');
      }
    };

    $$morph$range$$Morph.prototype.clear = function Morph$clear() {
      var node = this.setNode(this.domHelper.createComment(''));
      return node;
    };

    $$morph$range$$Morph.prototype.setText = function Morph$setText(text) {
      var firstNode = this.firstNode;
      var lastNode = this.lastNode;

      if (firstNode &&
          lastNode === firstNode &&
          firstNode.nodeType === 3) {
        firstNode.nodeValue = text;
        return firstNode;
      }

      return this.setNode(
        text ? this.domHelper.createTextNode(text) : this.domHelper.createComment('')
      );
    };

    $$morph$range$$Morph.prototype.setNode = function Morph$setNode(newNode) {
      var firstNode, lastNode;
      switch (newNode.nodeType) {
        case 3:
          firstNode = newNode;
          lastNode = newNode;
          break;
        case 11:
          firstNode = newNode.firstChild;
          lastNode = newNode.lastChild;
          if (firstNode === null) {
            firstNode = this.domHelper.createComment('');
            newNode.appendChild(firstNode);
            lastNode = firstNode;
          }
          break;
        default:
          firstNode = newNode;
          lastNode = newNode;
          break;
      }

      this.setRange(firstNode, lastNode);

      return newNode;
    };

    $$morph$range$$Morph.prototype.setRange = function (firstNode, lastNode) {
      var previousFirstNode = this.firstNode;
      if (previousFirstNode !== null) {

        var parentNode = previousFirstNode.parentNode;
        if (parentNode !== null) {
          $$morph$range$utils$$insertBefore(parentNode, firstNode, lastNode, previousFirstNode);
          $$morph$range$utils$$clear(parentNode, previousFirstNode, this.lastNode);
        }
      }

      this.firstNode = firstNode;
      this.lastNode = lastNode;

      if (this.parentMorphList) {
        this._syncFirstNode();
        this._syncLastNode();
      }
    };

    $$morph$range$$Morph.prototype.destroy = function Morph$destroy() {
      this.unlink();

      var firstNode = this.firstNode;
      var lastNode = this.lastNode;
      var parentNode = firstNode && firstNode.parentNode;

      this.firstNode = null;
      this.lastNode = null;

      $$morph$range$utils$$clear(parentNode, firstNode, lastNode);
    };

    $$morph$range$$Morph.prototype.unlink = function Morph$unlink() {
      var parentMorphList = this.parentMorphList;
      var previousMorph = this.previousMorph;
      var nextMorph = this.nextMorph;

      if (previousMorph) {
        if (nextMorph) {
          previousMorph.nextMorph = nextMorph;
          nextMorph.previousMorph = previousMorph;
        } else {
          previousMorph.nextMorph = null;
          parentMorphList.lastChildMorph = previousMorph;
        }
      } else {
        if (nextMorph) {
          nextMorph.previousMorph = null;
          parentMorphList.firstChildMorph = nextMorph;
        } else if (parentMorphList) {
          parentMorphList.lastChildMorph = parentMorphList.firstChildMorph = null;
        }
      }

      this.parentMorphList = null;
      this.nextMorph = null;
      this.previousMorph = null;

      if (parentMorphList && parentMorphList.mountedMorph) {
        if (!parentMorphList.firstChildMorph) {
          // list is empty
          parentMorphList.mountedMorph.clear();
          return;
        } else {
          parentMorphList.firstChildMorph._syncFirstNode();
          parentMorphList.lastChildMorph._syncLastNode();
        }
      }
    };

    $$morph$range$$Morph.prototype.setHTML = function(text) {
      var fragment = this.domHelper.parseHTML(text, this.contextualElement);
      return this.setNode(fragment);
    };

    $$morph$range$$Morph.prototype.setMorphList = function Morph$appendMorphList(morphList) {
      morphList.mountedMorph = this;
      this.clear();

      var originalFirstNode = this.firstNode;

      if (morphList.firstChildMorph) {
        this.firstNode = morphList.firstChildMorph.firstNode;
        this.lastNode = morphList.lastChildMorph.lastNode;

        var current = morphList.firstChildMorph;

        while (current) {
          var next = current.nextMorph;
          current.insertBeforeNode(originalFirstNode, null);
          current = next;
        }
        originalFirstNode.parentNode.removeChild(originalFirstNode);
      }
    };

    $$morph$range$$Morph.prototype._syncFirstNode = function Morph$syncFirstNode() {
      var morph = this;
      var parentMorphList;
      while (parentMorphList = morph.parentMorphList) {
        if (parentMorphList.mountedMorph === null) {
          break;
        }
        if (morph !== parentMorphList.firstChildMorph) {
          break;
        }
        if (morph.firstNode === parentMorphList.mountedMorph.firstNode) {
          break;
        }

        parentMorphList.mountedMorph.firstNode = morph.firstNode;

        morph = parentMorphList.mountedMorph;
      }
    };

    $$morph$range$$Morph.prototype._syncLastNode = function Morph$syncLastNode() {
      var morph = this;
      var parentMorphList;
      while (parentMorphList = morph.parentMorphList) {
        if (parentMorphList.mountedMorph === null) {
          break;
        }
        if (morph !== parentMorphList.lastChildMorph) {
          break;
        }
        if (morph.lastNode === parentMorphList.mountedMorph.lastNode) {
          break;
        }

        parentMorphList.mountedMorph.lastNode = morph.lastNode;

        morph = parentMorphList.mountedMorph;
      }
    };

    $$morph$range$$Morph.prototype.insertBeforeNode = function Morph$insertBeforeNode(parentNode, refNode) {
      $$morph$range$utils$$insertBefore(parentNode, this.firstNode, this.lastNode, refNode);
    };

    $$morph$range$$Morph.prototype.appendToNode = function Morph$appendToNode(parentNode) {
      $$morph$range$utils$$insertBefore(parentNode, this.firstNode, this.lastNode, null);
    };

    var $$morph$range$$default = $$morph$range$$Morph;

    (function (root, factory) {
      if (typeof define === 'function' && define.amd) {
        define([], factory);
      } else if (typeof exports === 'object') {
        module.exports = factory();
      } else {
        root.Morph = factory();
      }
    }(this, function () {
      return $$morph$range$$default;
    }));
}).call(this);

//# sourceMappingURL=morph-range.js.map