(function() {
    "use strict";
    var qunitjs$$QUnit = (function (root){
      if (typeof require === "function") {
        return require('qunitjs');
      } else {
        return root.QUnit;
      }
    }(this));

    var qunitjs$$default = qunitjs$$QUnit;
    var morph$range$morph$list$$MorphList = (function (root){
      if (typeof require === "function") {
        return require('./morph-range/morph-list');
      } else {
        return root.MorphList;
      }
    }(this));

    var morph$range$morph$list$$default = morph$range$morph$list$$MorphList;
    var morph$range$$Morph = (function (root){
      if (typeof require === "function") {
        return require('./morph-range');
      } else {
        return root.Morph;
      }
    }(this));

    var morph$range$$default = morph$range$$Morph;
    var simple$dom$$SimpleDOM = (function (root){
      if (typeof require === "function") {
        return require('simple-dom');
      } else {
        return root.SimpleDOM;
      }
    }(this));

    var simple$dom$$default = simple$dom$$SimpleDOM;
    var simple$html$tokenizer$$HTML5Tokenizer = (function (root){
      if (typeof require === "function") {
        return require('simple-html-tokenizer');
      } else {
        return root.HTML5Tokenizer;
      }
    }(this));

    var simple$html$tokenizer$$default = simple$html$tokenizer$$HTML5Tokenizer;
    var support$$isArray = (function () {
      if (Array.isArray) {
        return Array.isArray;
      }
      return function isArray(arg) {
        return Object.prototype.toString.call(arg) === '[object Array]';
      };
    }());

    var support$$document = (function (root){
      if (root.document) {
        return root.document;
      }
      return new simple$dom$$default.Document();
    }(this));

    var support$$parser = new simple$dom$$default.HTMLParser(simple$html$tokenizer$$default.tokenize, support$$document, simple$dom$$default.voidMap);

    var support$$serializer = new simple$dom$$default.HTMLSerializer(simple$dom$$default.voidMap);

    function support$$element(tagName, attrs) {
      var el = support$$document.createElement(tagName);
      var i = 1;
      if (typeof attrs === 'object' && typeof attrs.nodeType === 'undefined') {
        i = 2;
        for (var key in attrs) {
          el.setAttribute(key, attrs[key]);
        }
      }

      for (; i<arguments.length; i++) {
        var node = arguments[i];
        if (typeof node === 'string') {
          node = support$$text(node);
        }
        el.appendChild(node);
      }
      return el;
    }

    function support$$fragment() {
      var frag = support$$document.createDocumentFragment();
      for (var i=0; i<arguments.length; i++) {
        frag.appendChild(arguments[i]);
      }
      return frag;
    }

    function support$$text(s) {
      return support$$document.createTextNode(s || '');
    }

    function support$$comment(s) {
      return support$$document.createComment(s || '');
    }

    function support$$domHelper() {
      return new support$$DOMHelper(support$$document, support$$parser);
    }

    function support$$parseHTML(html) {
      return support$$parser.parse(html);
    }

    function support$$toHTML(node) {
      if (support$$isArray(node)) {
        var nodes = node;
        var buffer = '';
        for (var i=0; i<nodes.length; i++) {
          buffer += support$$serializer.serialize(nodes[i]);
        }
        return buffer;
      }
      return support$$serializer.serialize(node);
    }

    qunitjs$$default.assert.equalHTML = function ( nodes, expected, message ) {
      var actual = support$$toHTML(nodes);
      var result = actual === expected;
      this.push(result, actual, expected, message);
    };

    function support$$DOMHelper(document, parser) {
      this.document = document;
      this.parser = parser;
    }

    support$$DOMHelper.prototype = {
      createComment: function (text) {
        return this.document.createComment(text);
      },
      createTextNode: function (text) {
        return this.document.createTextNode(text);
      },
      parseHTML: function (text, context) {
        return this.parser.parse(text, context);
      }
    };

    qunitjs$$default.module('ported list tests');

    qunitjs$$default.test('insertBeforeMorph adds a child morph and updates its parentMorph', function (assert) {
      var parentMorph = new morph$range$$default(support$$domHelper());
      var list = new morph$range$morph$list$$default();

      parentMorph.setMorphList(list);

      var insertion = support$$comment();

      var frag = support$$fragment(insertion);

      parentMorph.setContent(insertion);

      assert.equalHTML(frag, '<!---->', 'initial');

      var a = support$$element('p', 'A');
      var b = support$$element('p', 'B');
      var c = support$$element('p', 'C');
      var d = support$$element('p', 'D');

      var aMorph = new morph$range$$default(parentMorph.domHelper);
      var bMorph = new morph$range$$default(parentMorph.domHelper);
      var cMorph = new morph$range$$default(parentMorph.domHelper);
      var dMorph = new morph$range$$default(parentMorph.domHelper);

      aMorph.setContent(a);
      bMorph.setContent(b);
      cMorph.setContent(c);
      dMorph.setContent(d);

      // append when there is no list
      list.insertBeforeMorph(bMorph, null);

      assert.equalHTML(frag, '<p>B</p>', 'append B');
      assert.strictEqual(parentMorph.firstNode, b);
      assert.strictEqual(parentMorph.lastNode, b);
      assert.strictEqual(list.firstChildMorph, bMorph, 'firstChildMorph to equal B morph');
      assert.strictEqual(list.lastChildMorph, bMorph, 'lastChildMorph to equal B morph');

      // append when there is one item
      list.insertBeforeMorph(dMorph, null);

      assert.equalHTML(frag, '<p>B</p><p>D</p>', 'append D');
      assert.strictEqual(parentMorph.firstNode, b);
      assert.strictEqual(parentMorph.lastNode, d);
      assert.strictEqual(list.firstChildMorph, bMorph, 'firstChildMorph to be unchanged');
      assert.strictEqual(list.lastChildMorph, dMorph, 'lastChildMorph to change to D morph');

      // insert before lastChildMorph
      list.insertBeforeMorph(cMorph, dMorph);
      assert.equalHTML(frag, '<p>B</p><p>C</p><p>D</p>', 'insert C before D');

      assert.strictEqual(parentMorph.firstNode, b);
      assert.strictEqual(parentMorph.lastNode, d);
      assert.strictEqual(list.firstChildMorph, bMorph, 'firstChildMorph to be unchanged');
      assert.strictEqual(list.lastChildMorph, dMorph, 'lastChildMorph to be unchanged');

      // insert before firstChildMorph
      list.insertBeforeMorph(aMorph, bMorph);
      assert.equalHTML(frag, '<p>A</p><p>B</p><p>C</p><p>D</p>', 'insert A before B');
      assert.strictEqual(parentMorph.firstNode, a);
      assert.strictEqual(parentMorph.lastNode, d);
      assert.strictEqual(list.firstChildMorph, aMorph, 'firstChildMorph to change to A morph');
      assert.strictEqual(list.lastChildMorph, dMorph, 'lastChildMorph to be unchanged');
    });



    qunitjs$$default.test('insertBeforeMorph inserts correctly when appending to the morph', function (assert) {
      var dom = support$$domHelper();
      var parentMorph = new morph$range$$default(dom);
      var list = new morph$range$morph$list$$default();

      var insertion = support$$comment();

      var a = support$$element('p', 'A');
      var b = support$$element('p', 'B');

      var frag = support$$fragment(a, insertion, b);

      parentMorph.setContent(insertion);
      parentMorph.setMorphList(list);

      var childMorph = new morph$range$$default(dom);
      childMorph.setContent('Y');
      list.insertBeforeMorph(childMorph, null);
      assert.equalHTML(frag, '<p>A</p>Y<p>B</p>', 'appended content correctly');

      var refMorph = childMorph;
      childMorph = new morph$range$$default(dom);
      childMorph.setContent('W');
      list.insertBeforeMorph(childMorph, refMorph);
      assert.equalHTML(frag, '<p>A</p>WY<p>B</p>', 'prepended content correctly');

      childMorph = new morph$range$$default(dom);
      childMorph.setContent('X');
      list.insertBeforeMorph(childMorph, refMorph);
      assert.equalHTML(frag, '<p>A</p>WXY<p>B</p>', 'inserted content correctly');

      childMorph = new morph$range$$default(dom);
      childMorph.setContent('Z');
      list.insertBeforeMorph(childMorph, null);
      assert.equalHTML(frag, '<p>A</p>WXYZ<p>B</p>', 'inserted content correctly');
    });

    qunitjs$$default.test('insertContentBeforeMorph', function (assert) {
      var parentMorph = new morph$range$$default(support$$domHelper());
      var list = new morph$range$morph$list$$default();

      var insertion = support$$comment();

      var frag = support$$fragment(insertion);

      parentMorph.setContent(insertion);
      parentMorph.setMorphList(list);

      var aMorph = new morph$range$$default(parentMorph.domHelper);
      var bMorph = new morph$range$$default(parentMorph.domHelper);
      var cMorph = new morph$range$$default(parentMorph.domHelper);
      var dMorph = new morph$range$$default(parentMorph.domHelper);

      aMorph.setContent('A');
      bMorph.setContent('B');
      cMorph.setContent('C');
      dMorph.setContent('D');

      list.insertBeforeMorph(bMorph, null);
      list.insertBeforeMorph(dMorph, null);
      list.insertBeforeMorph(cMorph, dMorph);
      list.insertBeforeMorph(aMorph, bMorph);

      assert.equalHTML(frag, 'ABCD', 'inserted content correctly');
      assert.equal(parentMorph.firstNode.nodeValue, 'A');
      assert.equal(parentMorph.lastNode.nodeValue, 'D');

      var a = support$$element('p', 'A');
      var b = support$$element('p', 'B');
      var c = support$$element('p', 'C');
      var d = support$$element('p', 'D');

      aMorph.setContent(a);

      assert.equalHTML(frag, '<p>A</p>BCD', 'changed list through returned A morph');
      assert.strictEqual(parentMorph.firstNode, a);
      assert.equal(parentMorph.lastNode.nodeValue, 'D');

      bMorph.setContent(b);

      assert.equalHTML(frag, '<p>A</p><p>B</p>CD', 'changed list through returned B morph');

      cMorph.setContent(c);

      assert.equalHTML(frag, '<p>A</p><p>B</p><p>C</p>D', 'changed list through returned C morph');

      dMorph.setContent(d);

      assert.equalHTML(frag, '<p>A</p><p>B</p><p>C</p><p>D</p>', 'changed list through returned D morph');

      assert.strictEqual(parentMorph.firstNode, a);
      assert.strictEqual(parentMorph.lastNode, d);
      assert.strictEqual(list.firstChildMorph, aMorph, 'firstChildMorph to change to A morph');
      assert.strictEqual(list.lastChildMorph, dMorph, 'lastChildMorph to change to D morph');

      cMorph.destroy();

      assert.equalHTML(frag, '<p>A</p><p>B</p><p>D</p>', 'C was destroyed');
      assert.strictEqual(list.firstChildMorph, aMorph, 'firstChildMorph unchanged');
      assert.strictEqual(list.lastChildMorph, dMorph, 'lastChildMorph unchanged');
      assert.strictEqual(dMorph.previousMorph, bMorph, 'D morph previousMorph is B');
      assert.strictEqual(bMorph.nextMorph, dMorph, 'B morph nextMorph is D');

      assert.strictEqual(cMorph.parentMorphList, null);
      assert.strictEqual(cMorph.firstNode, null);
      assert.strictEqual(cMorph.lastNode, null);
      assert.strictEqual(c.parentNode, null);

      aMorph.destroy();

      assert.equalHTML(frag, '<p>B</p><p>D</p>', 'A was destroyed');
      assert.strictEqual(parentMorph.firstNode, b);
      assert.strictEqual(list.firstChildMorph, bMorph, 'firstChildMorph is B');

      assert.strictEqual(aMorph.parentMorphList, null);
      assert.strictEqual(aMorph.firstNode, null);
      assert.strictEqual(aMorph.lastNode, null);
      assert.strictEqual(a.parentNode, null);

      dMorph.destroy();

      assert.equalHTML(frag, '<p>B</p>', 'D was destroyed');
      assert.strictEqual(parentMorph.lastNode, b);
      assert.strictEqual(list.lastChildMorph, bMorph, 'firstChildMorph is B');

      assert.strictEqual(dMorph.parentMorphList, null);
      assert.strictEqual(dMorph.firstNode, null);
      assert.strictEqual(dMorph.lastNode, null);
      assert.strictEqual(d.parentNode, null);

      bMorph.destroy();

      assert.strictEqual(bMorph.parentMorphList, null);
      assert.strictEqual(bMorph.firstNode, null);
      assert.strictEqual(bMorph.lastNode, null);
      assert.strictEqual(d.parentNode, null);

      assert.strictEqual(list.firstChildMorph, null, 'firstChildMorph is null');
      assert.strictEqual(list.lastChildMorph, null, 'lastChildMorph is null');

      // insert the fragment
      frag = support$$fragment(frag);

      aMorph = new morph$range$$default(parentMorph.domHelper);
      aMorph.setNode(a);

      // ensure list still works
      list.insertBeforeMorph(aMorph, null);

      assert.equalHTML(frag, '<p>A</p>', 'detected insertion after clear');
    });

    qunitjs$$default.config.autostart = false;

    qunitjs$$default.module('MorphList tests', {
      setup: test$morph$list$test$$commonSetup
    });

    var test$morph$list$test$$dom, test$morph$list$test$$list;

    function test$morph$list$test$$commonSetup() {
      test$morph$list$test$$dom = support$$domHelper();
      test$morph$list$test$$list = new morph$range$morph$list$$default(test$morph$list$test$$dom);
    }

    function test$morph$list$test$$assertChildMorphs(assert, list, expectedMorphs) {
      var actual = [];
      var current = list.firstChildMorph;
      var last = null;

      while (current) {
        actual.push(current.label);

        if (last) {
          assert.strictEqual(current.previousMorph.label, last.label,
                             "expected " + current.label + "'s previous to be " + last.label);
        }

        assert.ok(current.parentMorphList === list, "the morphs have the list as the parent");

        last = current;
        current = current.nextMorph;
      }

      expectedMorphs = expectedMorphs.map(function(m) {
        return m.label;
      });

      assert.deepEqual(actual, expectedMorphs);
    }

    function test$morph$list$test$$assertOrphanedMorphs(assert, expectedMorphs) {
      for (var i=0, l=expectedMorphs.length; i<l; i++) {
        var current = expectedMorphs[i];

        assert.ok(current.parentMorphList === null, current.label + " should not have a parent");
        assert.ok(current.nextMorph === null, current.label + " should not have a nextMorph");
        assert.ok(current.previousMorph === null, current.label + " should not have a previousMorph");
      }
    }

    function test$morph$list$test$$morph(label) {
      var m = new morph$range$$default.empty(test$morph$list$test$$dom);
      m.label = label;
      return m;
    }

    qunitjs$$default.test("can create a MorphList", function(assert) {
      assert.strictEqual(test$morph$list$test$$list.mountedMorph, null);
      assert.strictEqual(test$morph$list$test$$list.firstChildMorph, null);
      assert.strictEqual(test$morph$list$test$$list.lastChildMorph, null);
    });

    qunitjs$$default.test("can append a Morph into a MorphList using insertBefore", function(assert) {
      var morph1 = test$morph$list$test$$morph("morph1");

      test$morph$list$test$$list.insertBeforeMorph(morph1, null);


      test$morph$list$test$$assertChildMorphs(assert, test$morph$list$test$$list, [ morph1 ]);
    });

    qunitjs$$default.test("can prepend a Morph into a MorphList", function(assert) {
      var morph2 = test$morph$list$test$$morph("morph2");
      var morph1 = test$morph$list$test$$morph("morph1");

      test$morph$list$test$$list.appendMorph(morph2);
      test$morph$list$test$$list.insertBeforeMorph(morph1, morph2);

      // TODO Deal with DOM

      test$morph$list$test$$assertChildMorphs(assert, test$morph$list$test$$list, [ morph1, morph2 ]);
    });

    qunitjs$$default.test("can insert a Morph into the middle of a MorphList", function(assert) {
      var morph3 = test$morph$list$test$$morph("morph3");
      var morph2 = test$morph$list$test$$morph("morph2");
      var morph1 = test$morph$list$test$$morph("morph1");

      test$morph$list$test$$list.appendMorph(morph1);
      test$morph$list$test$$list.appendMorph(morph3);
      test$morph$list$test$$list.insertBeforeMorph(morph2, morph3);

      // TODO Deal with DOM

      test$morph$list$test$$assertChildMorphs(assert, test$morph$list$test$$list, [ morph1, morph2, morph3 ]);
    });

    qunitjs$$default.test("can move a morph from one list to another", function(assert) {
      var list2 = new morph$range$morph$list$$default(test$morph$list$test$$dom);

      var morph1 = test$morph$list$test$$morph("morph1");

      test$morph$list$test$$list.appendMorph(morph1);
      list2.appendMorph(morph1);

      test$morph$list$test$$assertChildMorphs(assert, test$morph$list$test$$list, [ ]);
      test$morph$list$test$$assertChildMorphs(assert, list2, [ morph1 ]);
    });

    qunitjs$$default.test("can remove the only morph in a MorphList", function(assert) {
      var morph1 = test$morph$list$test$$morph("morph1");

      test$morph$list$test$$list.appendMorph(morph1);
      test$morph$list$test$$list.removeChildMorph(morph1);

      test$morph$list$test$$assertChildMorphs(assert, test$morph$list$test$$list, [ ]);
      test$morph$list$test$$assertOrphanedMorphs(assert, [ morph1 ]);
    });

    qunitjs$$default.test("can remove the first morph in a MorphList", function(assert) {
      var morph1 = test$morph$list$test$$morph("morph1");
      var morph2 = test$morph$list$test$$morph("morph2");

      test$morph$list$test$$list.appendMorph(morph1);
      test$morph$list$test$$list.appendMorph(morph2);
      test$morph$list$test$$list.removeChildMorph(morph1);

      test$morph$list$test$$assertChildMorphs(assert, test$morph$list$test$$list, [ morph2 ]);
      test$morph$list$test$$assertOrphanedMorphs(assert, [ morph1 ]);
    });

    qunitjs$$default.test("can remove the last morph in a MorphList", function(assert) {
      var morph1 = test$morph$list$test$$morph("morph1");
      var morph2 = test$morph$list$test$$morph("morph2");

      test$morph$list$test$$list.appendMorph(morph1);
      test$morph$list$test$$list.appendMorph(morph2);
      test$morph$list$test$$list.removeChildMorph(morph2);

      test$morph$list$test$$assertChildMorphs(assert, test$morph$list$test$$list, [ morph1 ]);
      assert.ok(morph2.parentMorphList === null, "A removed morph doesn't still have its old parent");
    });

    qunitjs$$default.test("can remove a middle morph in a MorphList", function(assert) {
      var morph1 = test$morph$list$test$$morph("morph1");
      var morph2 = test$morph$list$test$$morph("morph2");
      var morph3 = test$morph$list$test$$morph("morph3");

      test$morph$list$test$$list.appendMorph(morph1);
      test$morph$list$test$$list.appendMorph(morph2);
      test$morph$list$test$$list.appendMorph(morph3);
      test$morph$list$test$$list.removeChildMorph(morph2);

      test$morph$list$test$$assertChildMorphs(assert, test$morph$list$test$$list, [ morph1, morph3 ]);
      test$morph$list$test$$assertOrphanedMorphs(assert, [ morph2 ]);
    });

    qunitjs$$default.test("can clear the only morph in a MorphList", function(assert) {
      var morph1 = test$morph$list$test$$morph("morph1");

      test$morph$list$test$$list.appendMorph(morph1);
      test$morph$list$test$$list.clear();

      test$morph$list$test$$assertChildMorphs(assert, test$morph$list$test$$list, [ ]);
      test$morph$list$test$$assertOrphanedMorphs(assert, [ morph1 ]);
    });

    qunitjs$$default.test("can clear two morphs in a MorphList", function(assert) {
      var morph1 = test$morph$list$test$$morph("morph1");
      var morph2 = test$morph$list$test$$morph("morph2");

      test$morph$list$test$$list.appendMorph(morph1);
      test$morph$list$test$$list.appendMorph(morph2);
      test$morph$list$test$$list.clear();

      test$morph$list$test$$assertChildMorphs(assert, test$morph$list$test$$list, [ ]);
      test$morph$list$test$$assertOrphanedMorphs(assert, [ morph1, morph2 ]);
    });

    qunitjs$$default.test("can remove three morphs in a MorphList", function(assert) {
      var morph1 = test$morph$list$test$$morph("morph1");
      var morph2 = test$morph$list$test$$morph("morph2");
      var morph3 = test$morph$list$test$$morph("morph3");

      test$morph$list$test$$list.appendMorph(morph1);
      test$morph$list$test$$list.appendMorph(morph2);
      test$morph$list$test$$list.appendMorph(morph3);
      test$morph$list$test$$list.clear();

      test$morph$list$test$$assertChildMorphs(assert, test$morph$list$test$$list, [ ]);
      test$morph$list$test$$assertOrphanedMorphs(assert, [ morph1, morph2, morph3 ]);
    });

    //QUnit.test("can append a MorphList into a MorphList", function(assert) {
    //var list2 = new MorphList(dom);
    //list2.label = "list2";

    //list.appendMorph(list2);

    //assertChildMorphs(assert, list, [ list2 ]);
    //});

    var test$morph$list$test$$frag, test$morph$list$test$$root;

    qunitjs$$default.module("MorphList DOM Manipulation tests", {
      setup: function() {
        test$morph$list$test$$commonSetup();
        test$morph$list$test$$domSetup();
      }
    });

    function test$morph$list$test$$domSetup() {
      test$morph$list$test$$frag = support$$document.createDocumentFragment();
      test$morph$list$test$$root = test$morph$list$test$$morph("root");
      test$morph$list$test$$root.appendToNode(test$morph$list$test$$frag);
      test$morph$list$test$$root.setMorphList(test$morph$list$test$$list);
    }

    function test$morph$list$test$$assertInvariants(assert) {
      assert.strictEqual(test$morph$list$test$$frag.firstChild, test$morph$list$test$$root.firstNode, "invariant: the fragment's first child is the root's first node");
      assert.strictEqual(test$morph$list$test$$frag.lastChild, test$morph$list$test$$root.lastNode, "invariant: the fragment's last child is the root's last node");
    }

    qunitjs$$default.test("appending a morph updates the DOM representation", function(assert) {
      var morph1 = test$morph$list$test$$morph("morph1");
      morph1.setNode(support$$text("hello"));

      test$morph$list$test$$list.appendMorph(morph1);
      test$morph$list$test$$assertInvariants(assert);

      assert.equalHTML(test$morph$list$test$$frag, "hello");

      var morph2 = test$morph$list$test$$morph("morph2");
      morph2.setNode(support$$text(" world"));
      test$morph$list$test$$list.appendMorph(morph2);
      test$morph$list$test$$assertInvariants(assert);

      assert.equalHTML(test$morph$list$test$$frag, "hello world");
    });

    qunitjs$$default.test("prepending a morph updates the DOM representation", function(assert) {
      var morph1 = test$morph$list$test$$morph("morph1");
      morph1.setNode(support$$text("world"));

      test$morph$list$test$$list.appendMorph(morph1);
      test$morph$list$test$$assertInvariants(assert);

      assert.equalHTML(test$morph$list$test$$frag, "world");

      var morph2 = test$morph$list$test$$morph("morph2");
      morph2.setNode(support$$text("hello "));
      test$morph$list$test$$list.insertBeforeMorph(morph2, morph1);
      test$morph$list$test$$assertInvariants(assert);

      assert.equalHTML(test$morph$list$test$$frag, "hello world");
    });

    qunitjs$$default.test("removing the last morph makes the mount point empty again", function(assert) {
      var morph1 = test$morph$list$test$$morph("morph1");
      morph1.setNode(support$$text("hello world"));
      assert.equalHTML(test$morph$list$test$$frag, "<!---->");

      test$morph$list$test$$list.appendMorph(morph1);
      test$morph$list$test$$assertInvariants(assert);
      assert.equalHTML(test$morph$list$test$$frag, "hello world");

      test$morph$list$test$$list.removeChildMorph(morph1);
      test$morph$list$test$$assertInvariants(assert);
      assert.equalHTML(test$morph$list$test$$frag, "<!---->");
    });

    qunitjs$$default.test("multiple nestings is allowed", function(assert) {
      var list2 = new morph$range$morph$list$$default(test$morph$list$test$$dom);
      list2.label = "list2";

      var middle = test$morph$list$test$$morph("middle");
      middle.setMorphList(list2);

      var content = test$morph$list$test$$morph("content");
      content.setNode(support$$text("hello world"));

      test$morph$list$test$$list.appendMorph(middle);

      list2.appendMorph(content);
      test$morph$list$test$$assertInvariants(assert);

      assert.equalHTML(test$morph$list$test$$frag, "hello world");

      list2.removeChildMorph(content);
      test$morph$list$test$$assertInvariants(assert);

      assert.equalHTML(test$morph$list$test$$frag, "<!---->");

      test$morph$list$test$$root.setNode(support$$text("hello world"));
      test$morph$list$test$$assertInvariants(assert);

      assert.equalHTML(test$morph$list$test$$frag, "hello world");
    });

    var test$morph$list$test$$list2, test$morph$list$test$$c1, test$morph$list$test$$c2, test$morph$list$test$$c3;

    qunitjs$$default.module("Recursively updating firstNode and lastNode", {
      setup: function() {
        test$morph$list$test$$commonSetup();
        test$morph$list$test$$domSetup();

        test$morph$list$test$$list2 = new morph$range$morph$list$$default(test$morph$list$test$$dom);
        test$morph$list$test$$list2.label = "list2";

        var middle = test$morph$list$test$$morph("middle");
        middle.setMorphList(test$morph$list$test$$list2);
        test$morph$list$test$$list.appendMorph(middle);

        test$morph$list$test$$c1 = test$morph$list$test$$morph("c1");
        test$morph$list$test$$c1.setNode(support$$text("c1"));

        test$morph$list$test$$c2 = test$morph$list$test$$morph("c2");
        test$morph$list$test$$c2.setNode(support$$text("c2"));

        test$morph$list$test$$c3 = test$morph$list$test$$morph("c3");
        test$morph$list$test$$c3.setNode(support$$text("c3"));

        test$morph$list$test$$list2.appendMorph(test$morph$list$test$$c1);
        test$morph$list$test$$list2.appendMorph(test$morph$list$test$$c2);
        test$morph$list$test$$list2.appendMorph(test$morph$list$test$$c3);
      }
    });

    qunitjs$$default.test("sanity checks", function(assert) {
      assert.equalHTML(test$morph$list$test$$frag, "c1c2c3");
      test$morph$list$test$$assertInvariants(assert);
    });

    qunitjs$$default.test("removing the first node updates firstNode", function(assert) {
      test$morph$list$test$$list2.removeChildMorph(test$morph$list$test$$c1);
      test$morph$list$test$$assertInvariants(assert);
      assert.equalHTML(test$morph$list$test$$frag, "c2c3");
    });

    qunitjs$$default.test("removing the last node updates lastNode", function(assert) {
      test$morph$list$test$$list2.removeChildMorph(test$morph$list$test$$c3);
      test$morph$list$test$$assertInvariants(assert);
      assert.equalHTML(test$morph$list$test$$frag, "c1c2");
    });

    qunitjs$$default.test("removing a middle node doesn't do anything", function(assert) {
      test$morph$list$test$$list2.removeChildMorph(test$morph$list$test$$c2);
      test$morph$list$test$$assertInvariants(assert);
      assert.equalHTML(test$morph$list$test$$frag, "c1c3");
    });

    qunitjs$$default.test("prepending a node updates firstNode", function(assert) {
      var c4 = test$morph$list$test$$morph("c4");
      c4.setNode(support$$text("c4"));

      test$morph$list$test$$list2.insertBeforeMorph(c4, test$morph$list$test$$c1);
      test$morph$list$test$$assertInvariants(assert);
      assert.equalHTML(test$morph$list$test$$frag, "c4c1c2c3");
    });

    qunitjs$$default.test("appending a node updates lastNode", function(assert) {
      var c4 = test$morph$list$test$$morph("c4");
      c4.setNode(support$$text("c4"));

      test$morph$list$test$$list2.appendMorph(c4);
      test$morph$list$test$$assertInvariants(assert);
      assert.equalHTML(test$morph$list$test$$frag, "c1c2c3c4");
    });

    qunitjs$$default.test("moving a morph from one list to another updates firstNode", function(assert) {
      var list3 = new morph$range$morph$list$$default(test$morph$list$test$$dom);
      var secondMiddle = test$morph$list$test$$morph("secondMiddle");
      secondMiddle.setMorphList(list3);

      test$morph$list$test$$list.appendMorph(secondMiddle);

      var morph1 = test$morph$list$test$$morph("morph1");

      test$morph$list$test$$list2.appendMorph(morph1);
      test$morph$list$test$$assertInvariants(assert);

      test$morph$list$test$$assertChildMorphs(assert, test$morph$list$test$$list2, [ test$morph$list$test$$c1, test$morph$list$test$$c2, test$morph$list$test$$c3, morph1 ]);

      list3.appendMorph(morph1);
      test$morph$list$test$$assertInvariants(assert);

      test$morph$list$test$$assertChildMorphs(assert, test$morph$list$test$$list2, [ test$morph$list$test$$c1, test$morph$list$test$$c2, test$morph$list$test$$c3 ]);
      test$morph$list$test$$assertChildMorphs(assert, list3, [ morph1 ]);
    });

    qunitjs$$default.module('Morph tests');

    qunitjs$$default.test('can construct a Morph', function (assert) {
      var m = new morph$range$$default(support$$domHelper());
      assert.ok(m, "this test is fine" );
    });

    qunitjs$$default.test('can setContent of a morph', function (assert) {
      var morph = new morph$range$$default(support$$domHelper());

      var insertion = support$$comment();

      morph.setContent(insertion);

      var frag = support$$fragment(
        support$$element('p', 'before ', insertion, ' after')
      );

      morph.setContent('Hello World');

      assert.equalHTML(frag, '<p>before Hello World after</p>', 'it updated');

      morph.clear();

      assert.equalHTML(frag, '<p>before <!----> after</p>', 'clear');

      frag = support$$fragment(frag);

      morph.setContent('Another test...');

      assert.equalHTML(frag, '<p>before Another test... after</p>', 'works after appending to a fragment');

      var el = support$$element('div', '\n', frag, '\n');

      morph.setContent('Again');

      assert.equalHTML(el, '<div>\n<p>before Again after</p>\n</div>', 'works after appending to an element');

      morph.setContent('');

      assert.equalHTML(el, '<div>\n<p>before  after</p>\n</div>', 'setting to empty');
    });

    qunitjs$$default.test('uses toHTML for SafeString support', function (assert) {
      var morph = new morph$range$$default(support$$domHelper());

      var insertion = support$$comment();

      morph.setContent(insertion);

      var frag = support$$fragment(
        support$$element('p', 'before ', insertion, ' after')
      );

      morph.setContent({
        toHTML: function() {
          return '<b>Hello World!</b>';
        }
      });

      assert.equalHTML(frag, '<p>before <b>Hello World!</b> after</p>', 'it updated');
    });

    qunitjs$$default.test('an object with a .string property does not assume SafeString', function (assert) {
      var morph = new morph$range$$default(support$$domHelper());

      var insertion = support$$comment();

      morph.setContent(insertion);

      var frag = support$$fragment(
        support$$element('p', 'before ', insertion, ' after')
      );

      morph.setContent({
        string: '<b>Hellow World!</b>',

        toString: function() {
          return '<b>Hellow World!</b>';
        }
      });

      assert.equalHTML(frag, '<p>before &lt;b&gt;Hellow World!&lt;/b&gt; after</p>', 'it updated');
    });

    qunitjs$$default.test("When destroying a morph, do not explode if a parentMorph does not exist", function(assert) {
      var dom = support$$domHelper();
      var morph = new morph$range$$default(dom);
      morph.clear();

      var morphFrag = support$$document.createDocumentFragment();
      morphFrag.appendChild(morph.firstNode);
      assert.strictEqual(morphFrag.firstChild, morph.firstNode);
      assert.strictEqual(morphFrag.lastChild, morph.lastNode);
      morph.destroy();
      assert.strictEqual(morphFrag.firstChild, null);
    });

    qunitjs$$default.test("When destroying a morph, do not explode if a parentNode does not exist", function(assert) {
      var dom = support$$domHelper();
      var morph = new morph$range$$default(dom);
      morph.destroy();
      assert.ok(true, "The test did not crash");
    });

    qunitjs$$default.test("When inserting a range into a morph, range is inserted in the correct order", function(assert) {
      var dom = support$$domHelper();
      var morph = new morph$range$$default(dom);
      var insertion = support$$comment();
      var el = support$$element('div', insertion);
      morph.setContent(insertion);
      var frag = support$$fragment(
        support$$element('p', 'a'),
        support$$element('p', 'b'),
        support$$element('p', 'c'),
        support$$element('p', 'd')
      );
      morph.setContent(frag);
      assert.equalHTML(el, '<div><p>a</p><p>b</p><p>c</p><p>d</p></div>', 'updates range in correct order');
    });
}).call(this);

//# sourceMappingURL=test.js.map